function extension(archivotxt,orden_ext)
% Sistemas de Comunicacin 2013 - Taller
% Instituto de Ingeniera Elctrica
%
% Nombre: extension
%
% Descripcin: Calcula la extension de una fuente, desplegando los valores
% de Entropia para las extensiones normalizadas de orden 2,4,8 (H(X^n)/n). 
% Calcula el codigo Huffman de $orden$ ingresado como parametro (2,4,8) y 
% codifica el archivotxt mediante dicho codigo. Calcula la tasa de compresion. 
% Se pueden utilizar los archivos de ejemplo: "decreto.txt", "mensaje1.txt" y
% "mensaje2.txt"
%
% 
% ::Sintxis::
%  
%   extension(archivotxt,orden_ext)
  
%      -->   archivotxt = 'decreto.txt'
%      -->   orden_ext = 4
%
% Parmetros: 
% archivotxt - Archivo ASCII a codificar
% orden_ext - Orden de la extensin de la fuente a codificar utilizando Huffman 
%

if nargin == 1
    orden_ext = 4;
end

punto=max(findstr(archivotxt,'.'));
if ~isempty(punto)
    archivo = archivotxt(1:punto-1);
   else
    archivo = archivotxt;
end

archivobin = [archivo,'.bin'];
archivocod = [archivo,'_huff.bin'];    
msgbin = texto2bin(archivotxt,archivobin);
disp(['Archivo en binario grabado en : ',archivobin])
disp(' ')

estad = estadistica(msgbin);
disp(['Entropa de la fuente H(A) : ',num2str(entropia(estad))])

for i = 1:3
  orden = 2^i;
  estad = estadistica(msgbin,orden);
  disp(['Entropa de la fuente H(A) extendida: ',num2str(orden),' veces : ',num2str(entropia(estad)/orden)])
end

orden = orden_ext;
lmsg  = length(msgbin);
resto = mod(lmsg,orden);
if resto
    msgbin = [msgbin;zeros(orden - resto,1)];
    lmsg = lmsg + orden - resto;
end
disp(' ')
estad = estadistica(msgbin,orden);
disp(['Calculando el codigo de Huffman de la fuente extendida de orden : ', num2str(orden)])
codigo = huffman(estad);


disp(' ')
disp('Codigo: ')
disp(codigo)

disp(' ')
disp('Codificando...')
disp(['Archivo codificado guardado en : ', archivocod])
msgCodif = codifica(msgbin,codigo);
bin2file(msgCodif,archivocod)

estad_huf = estadistica(msgCodif);
disp(['Entropa de la fuente Huffman codificada H(A) : ', num2str(entropia(estad_huf))])
disp(' ')
disp(['Tasa de Compresin 1- largo(msg)/largo(msg_codif) : ', num2str(1-length(msgCodif)/length(msgbin))])


%Lo meto al canal
%msgCodifC = CSBB(msgCodif,0.9);
msgCodifC = msgCodif;

%ahroa no necesito decodificar.
%disp(' ')
%disp('Decodificando...')
%msgbin2 = decodifica(msgCodifC,codigo);
%bin2texto(msgbin2,'sal.txt');
