function simulacion(archivotxt,p)
% Sistemas de Comunicacin 2013 - Taller
% Instituto de Ingeniera Elctrica
%
% Nombre: simulacion
%
% Descripcin: Simula el comportamiento de un canal con borrado. Ademas calcula 
% la entropia de la fuente (A) la de la fuente luego de ser transmitida por el 
% canal (B) y despliega en pantalla. Tambien se muestra la informacion mutua I(A;B)
% estimando la Capacidad del canal (C). Para simular el comportamiento de la fuente,
% se utiliza el mensaje $archivotxt$. La funcion tambien toma como parametro la 
% probabilidad de borrado del canal $p$. Como ejemplo se puede utilizar el archivo 
% "decreto.txt". El punto particular resaltado en los graficos corresponde a la 
% estadistica tomada del archivo: archivotxt. Para el correcto funcionamiento debe 
% implementarse la funcion "canal.m" (ver instructivo).
% 
% ::Sintaxis::
%
%   simulacion(archivotxt,p)
%   
%      -->    archivotxt = 'decreto.txt'
%      -->    p = 0.01
%
% Par�metros: 
% archivotxt - Archivo ASCII a codificar
% p - Probabilidad de que un simbolo sea borrado al transmitirlo por el canal. 
%


disp(['Cargango el archivo: ', archivotxt]) 
disp(' ')
[msgbin,msgtxt] = texto2bin(archivotxt);
Pa = estadistica(msgbin);

%Calculo Entropia y demas yerbas.
Ha = entropia(Pa);
disp(['Entropa H(A): ',num2str(Ha)])
disp(' ')

a = [0:0.01:1];
pA = [a; (1-a)];
hA = entropia(pA);
win = figure(1);
plot(a,hA,'b');
hold on
plot(Pa(1),Ha,'ro')
plot([0,Pa(1)],[Ha,Ha],'r-.')
plot([Pa(1) Pa(1)],[0 , Ha],'r-.')
title('Entropia H(A) en funcin de a');
xlabel('a: probabilidad de enviar uno de los smbolos');
ylabel('Entropa');

% Cualquier tecla para seguir.
disp(' ')
disp('En pausa');
pause
close(win);

%-----------------------------------------
 P =  [(1-p) p 0; 0 p (1-p)];
 pB = (pA'*P)'; 
 hB = entropia(pB);
 

 %pare el caso del mensaje
 msgbinB = canal(msgbin,p);

msgtxtB =  bin2texto(msgbinB,['C',archivotxt]);
disp(' ')
disp(['La salida del canal se grabrab en: ', 'C',archivotxt])
 
 Pb = estadistica(msgbinB);
 Hb = entropia(Pb); 	
 disp(['Probabilidad de borrado: ', num2str(p)]) 
 disp(['Entropa H(B): ',num2str(Hb)]) 
 
  win = figure(1);
  plot(a,hB)
  hold on
  plot(Pa(1),Hb,'o')
  plot([0;Pa(1)],[Hb;Hb],'-.')
  plot([Pa(1);Pa(1)],[zeros(size(p)) ; Hb],'-.')
  title(['Entropa H(B) en funcin de a,  p = ',num2str(p)]);
  xlabel('a: probabilidad de enviar uno de los smbolos');
  ylabel('Entropa');

% Cualquier tecla para seguir.
disp(' ')
disp('En pausa');
pause
close(win);


%--------------------------------------------------------


% Calculamos la entropia H(A/B).
%
% Calculamos la matriz de probabilidades conjuntas
%
%            | a.p    q.a      0  | 
%   p(a,b) = |                    |
%            | 0    q(1-a)  p(1-a)|
%
% En base a esto utizamos las funciones del toolbox


Pc = condconj(P,Pa);
[HAB HBA] = entcond(Pc);
Imut=infmut(Pc);

disp(' ')
disp(['H(A/B) : ',num2str(HBA)])
disp(['H(B/A) : ',num2str(HAB)])
disp(['I(A;B) : ',num2str(Imut)])

%--------------------------------------------
%Calculo la capacidad del canal
a = [0:0.01:1];
pA = [a; (1-a)];
[m,n] = size(pA);

for i=1:n
	Pc = condconj(P,pA(:,i));
	[hAB(i), hBA(i)] = entcond(Pc);
	imut(i)=infmut(Pc);
end

C = max(imut);
disp(['Capacidad del Canal C : ', num2str(C)])
%-------------------------------------------
 win = figure(1);
 
 plot(a,hAB(1,:),'r',a,hBA(1,:),'g',a,imut(1,:),'b');
 hold on
 
 plot(Pa(1),HAB,'ro')
 plot([0;Pa(1)],[HAB;HAB],'r-.')
 plot([Pa(1);Pa(1)],[zeros(size(p)) ; HAB],'r-.')
 
 plot(Pa(1),HBA,'go')
 plot([0;Pa(1)],[HBA;HBA],'g-.')
 plot([Pa(1);Pa(1)],[zeros(size(p)) ; HBA],'g-.')
 
 plot(Pa(1),Imut,'bo')
 plot([0;Pa(1)],[Imut;Imut],'b-.')
 plot([Pa(1);Pa(1)],[zeros(size(p)) ; Imut],'b-.')
 
 
 title('H(A/B), H(B/A) y Inf. mutua');
 xlabel('a: probabilidad de enviar uno de los smbolos');
 legend('H(B/A)','H(A/B)', 'Inf. mutua')
 
 hold off
 
 %text(0.5,0.6, 'H(A/B)');
 %text(0.5,0.9, 'H(B/A)');
 %text(0.5,0.3, 'Inf. mutua');

 % line(cap_canal(,1),cap_canal(1,2));


% Capacidad del canal: cap = p
% En los ejemplos
% Capacidad del canal (p = 0.25)

%disp(['Capacidad del Canal: ' num2str(cap_canal(1,2))])


% Cualquier tecla para seguir.
%disp('En pausa');
%pause
%close(win);



%Tasa de error
%en binario y en car�cteres
t_error_bits = tasa_error(msgbin,msgbinB);
t_error_car = tasa_error(msgtxt,msgtxtB);
disp(' ')
disp(['Tasa de error en bits: ',num2str(t_error_bits)])
disp(['Tasa de error en caracteres: ',num2str(t_error_car)])








